/*****************************************************************************
 Copyright  Newcastle University
 Author      Paul Coates
 *****************************************************************************/
#include <string.h>
#include <stdlib.h>
#include "router.h"
#include "ipaddress.h"
#include "interface.h"
#include "misc.h"

Router::Router()
{
	clear();
}

Router::~Router()
{
	Interface *in;
	Link *ln;

	for (ln=interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (Interface*) ln->item;
		delete in;
	}
}

void Router::clear()
{
	int i;

	x = rand()%400;
	y = rand()%300;
	w = h = 16;
	selected = 0;
	show = 1;
	type = 'u';
	visited = 0;
	find = 0;

	strcpy(name, "");
	last_seen = 0;
	strcpy(community, "");
	strcpy(location, "");
	strcpy(descr, "");
	strcpy(contact, "");
	status = 0;
	registered = 0;
	matrix = 0;
}

void Router::print()
{
	Interface *in;
	Link *ln;

	printf("name \"%s\", ", name);
	printf("type %c, num_interfaces %d\n", type, interfaces.size());

	for (ln=interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (Interface*) ln->item;
		in->print();
	}
}

void Router::save(FILE *f)
{
	int i, j;
	Interface *in;
	Link *ln;

	pingname.save(f);
	for (i=0; i<64; i++) fputc(name[i], f);
	write_time_t(f, last_seen);
	for (i=0; i<64; i++) fputc(location[i], f);
	for (i=0; i<64; i++) fputc(descr[i], f);
	for (i=0; i<64; i++) fputc(contact[i], f);
	for (i=0; i<64; i++) fputc(community[i], f);
	write_int(f, registered);
	write_int(f, matrix);
	fputc(type, f);
	write_int(f, interfaces.size());
	for (ln=interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (Interface*) ln->item;
		in->save(f);
	}
}

void Router::load(FILE *f)
{
	int i, j;
	Interface *in;

	pingname.load(f);
	for (i=0; i<64; i++) name[i] = fgetc(f); name[64] = '\0';
	last_seen = read_time_t(f);
	for (i=0; i<64; i++) location[i] = fgetc(f); location[64] = '\0';
	for (i=0; i<64; i++) descr[i] = fgetc(f); descr[64] = '\0';
	for (i=0; i<64; i++) contact[i] = fgetc(f); contact[64] = '\0';
	for (i=0; i<64; i++) community[i] = fgetc(f); community[64] = '\0';
	registered = read_int(f);
	matrix = read_int(f);
	type = fgetc(f);
	j = read_int(f);
	interfaces.clear();
	for (i=0; i<j; i++)
	{
		in = new Interface();
		in->load(f);
		in->router = this;
		interfaces.tag(in);
	}
}

void Router::match(char *string)
{
	char mac[16];
	char ip3[16];
	Interface *in;
	Link *ln, *ln2;
	IPAddress *ip2;

	find = 0;
	if (strlen(string)==0) return;

	if ((strlen(name)>0) && (lower_contains(name, string))) find = 1; else
	if ((strlen(location)>0) && (lower_contains(location, string))) find = 2; else
	if ((strlen(descr)>0) && (lower_contains(descr, string))) find = 3; else
	if ((strlen(contact)>0) && (lower_contains(contact, string))) find = 4;

	for (ln = interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (Interface*) ln->item;
		sprint_mac(in->mac, mac);
		if ((strlen(mac)>0) && (lower_contains(mac, string))) find = 5;
		for (ln2 = in->ip_addresses.get_head(); ln2!=NULL; ln2=ln2->next)
		{
			ip2 = (IPAddress*) ln2->item;
			ip2->sprint(ip3);
			if (contains(ip3, string)) find = 6;
		}
	}
}

void Router::savexml(xmlNodePtr xmlnode)
{
	xmlNodePtr newxml;
	char buf[256];
	Link *ln;
	Interface *in;

	newxml = xmlNewChild(xmlnode, NULL, (xmlChar*)"Device", NULL);
	pingname.sprint(buf);
	xmlSetProp(newxml, (xmlChar*)"pingname", (xmlChar*)buf);
	xmlSetProp(newxml, (xmlChar*)"name", (xmlChar*)name);
	make_string_from_time(last_seen, buf);
	xmlSetProp(newxml, (xmlChar*)"lastseen", (xmlChar*)buf);
	xmlSetProp(newxml, (xmlChar*)"location", (xmlChar*)location);
	xmlSetProp(newxml, (xmlChar*)"description", (xmlChar*)descr);
	xmlSetProp(newxml, (xmlChar*)"contact", (xmlChar*)contact);
	xmlSetProp(newxml, (xmlChar*)"community", (xmlChar*)community);
	sprintf(buf, "%d", registered);
	xmlSetProp(newxml, (xmlChar*)"registered", (xmlChar*)buf);
	sprintf(buf, "%d", matrix);
	xmlSetProp(newxml, (xmlChar*)"matrix", (xmlChar*)buf);
	sprintf(buf, "%c", type);
	xmlSetProp(newxml, (xmlChar*)"type", (xmlChar*)buf);

	for (ln=interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (Interface*) ln->item;
		in->savexml(newxml);
	}
}

void Router::loadxml(xmlNodePtr xmlnode)
{
	xmlChar *buf;
	Interface *in;

	// check this is a router node
	if (!xmlnode->name || strcmp((char*)xmlnode->name, "Device")!=0)
		return;

	// read router properties
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"pingname");
	pingname.set((char*)buf);	free(buf);
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"name");
	strcpy(name, (char*)buf);	free(buf);
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"lastseen");
	last_seen = make_time_from_string((char*)buf);	free(buf);
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"location");
	strcpy(location, (char*)buf);	free(buf);
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"description");
	strcpy(descr, (char*)buf);	free(buf);
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"contact");
	strcpy(contact, (char*)buf);	free(buf);
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"community");
	strcpy(community, (char*)buf);	free(buf);
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"registered");
	sscanf((char*)buf, "%d", &registered);	free(buf);
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"matrix");
	sscanf((char*)buf, "%d", &matrix);	free(buf);
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"type");
	if (buf!=NULL) type = buf[0];	free(buf);

	interfaces.clear();
	xmlnode = xmlnode->childs;
	while (xmlnode != NULL)
	{
		if (xmlnode->name!=NULL)
		{
			if (strcmp((char*)xmlnode->name, "Interface")==0)
			{
				in = new Interface();
				in->loadxml(xmlnode);
				in->router = this;
				interfaces.tag(in);
			}
		}
		xmlnode = xmlnode->next;
	}
}

